// module hu_trace.h
// Dr.E.Huckert 2005

// Versions:
// 13.03.2008 EH logging via UDP added, list based logging dropped

#ifndef HUTRACE_DEFINED
#define HUTRACE_DEFINED

#include <stdio.h>
#include <stdlib.h>
#include <sys/timeb.h>

#ifdef WIN32
// Windows here
#include <winsock2.h>
//#include <windows.h>
//#include <wx/msw/winundef.h>
#include <io.h>
//WSADATA WSAdata;
#else
// Linux here
#include <unistd.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <resolv.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/msg.h>
#include <netinet/in.h>
#endif

#ifdef USE_STL
#include <string>
#else
#include <wx/wx.h>
#include <wx/string.h>
#include <wx/textfile.h>
#endif

#define sock_t int

using namespace std;

class HU_TRACE
{
private:
  char *filename;            // for file based logging
  int ofd;                   // for file based logging
  char *udpServer;           // for UDP based logging
  int udpPort;               // for UDP based logging
  sock_t sock;
  unsigned long count;
  int error; 
  //
  void initialize(const char *fn, const char *udpserver, int port);
  int UDP_send(const char *msg);
  void cleanup(void);
  sock_t create_socket(int af, int type, int protocol);
  char *convert_binary_ip(char *binstr);
  int UDP_send(sock_t sock, 
               const char *data,   // data to be sent out
               int   size,  // no. of bytes to send
               const char *serverName,
               int  port);
  int UDP_send(const char *msg, int noBytes);
  char *getTime();

public:
  HU_TRACE();
  HU_TRACE(const char *fn);
  HU_TRACE(const char *udpserver, int udpport);
  ~HU_TRACE();
  void setLock();
  void unLock();
  void trace(const char *msg);
  void traceLF(const char *msg);
  void trace(const char *msg, const char *arg1);
  void trace(const char *msg, int arg1);
  void trace(const char *msg, int arg1, int arg2);
  void trace(const char *msg, int arg1, int arg2, int arg3);
void trace(const char *msg, int arg1, int arg2, int arg3, int arg4);
  void trace(const char *msg, unsigned int arg1);
  void trace(const char *msg, long arg1);
  void trace(const char *msg, float arg1);
  //void trace(DL_LIST *plist, const char *msg1, const char *msg2);
  void tracex(const char *msg, long arg1);
  void trace(const char *msg, unsigned long arg1);
  void setFileName(const char *fn);
  void trace(const wxString str);
  void trace(const wxString *pstr);
  void trace(const char *str1, const wxString str2);
  void trace(const char *str1, const wxString *pstr2);
  void trace(const char *str1, const char ch);
  void trace(const char *p1,   const wxString p2, int p3);
  void trace(const char *p1,   const wxString p2, const wxString p3);
  void trace(const char *p1,   const wxString p2,
                               int p3, int p4);
  void trace(const char *p1,   const wxString p2,
                               int p3, int p4, int p5);
};
#endif
