// module inifilewx.cpp
// Implements a simple (WINDOWS ini-file like) associative access
// to key-value pairs, where the keys have the form section+item
// Uses exclusively wxWidgets - no STL!
// Copyright Private Copyright Dr.E.Huckert 10-2002 
// Tested with MSC V6.0 and Digital Mars C++
//
// Versions:
// 2004-11-16 error in tolower_buf fixed
// 2005-04-13 method write() corrected
// 2014-08-02 unused variables eliminated, compiled for g++
//                       methods tolower_buf() eliminated
// 08.08.2014 Verwendung der Module hu_str.cpp und dvli.cpp eliminiert
//                     Verwendet kein STL mehr - reine wxWidgets Klassen fuer Container und Strings

#ifndef DMC
#include <iostream>
#else
#include <iostream.h>  //  for Digital Mars compiler - note compile symbol "DMC"
#endif
#include <stdio.h>
#include <ctype.h>

#include "inifilewx.h"

using namespace std;

// ---------------------------------------------------------------
// Constructor
IniFileWx::IniFileWx()
{
  origin_fn = "";
  this->pTrace  = NULL;
}

// ---------------------------------------------------------------
// Destructor
IniFileWx::~IniFileWx()
{
  this->mmap.clear();
}

// ----------------------------------------------------------------
void IniFileWx::setLogging(HU_TRACE *pt)
{
  this->pTrace = pt;
}

// ---------------------------------------------------------------
void IniFileWx::clean_buf(char *buf)
{
    int n;
 if (buf == NULL)
   return;
 for (n=0; buf[n] != 0; n++)
 {
   if (buf[n] == '\n') buf[n] = 0;
   if (buf[n] == '\r') buf[n] = 0;
 }
}   // end clear_buf()

// ---------------------------------------------------------------
// Allocates and fills a key
// Returns the generated key
//         may return NULL upon error
// Sample key: "[general]hostname"
wxString IniFileWx::makeKey(const wxString & section, 
                            const wxString & item)
{
  wxString key;
  //
  key = "[";
  key.append(section);
  key.append("]");
  key.append(item); // result is "[section]item", also "[]item"
  return key;
}   // end IniFileWx::makeKey()

// ---------------------------------------------------------------
// Fill the dictionary from a Windows-ini like file
// Returns: the number of entries if successful
//          ==0 if error
long IniFileWx::load(const char *filename)
{
  FILE *ifd = NULL;
  long no_entries = 0;
  char buf[512];
  char k[256];
  char *caddr;
  char section[256];
  int  m, ret;
  unsigned int n;
  wxString secStr, itStr, valStr;
  //
  if (this->pTrace != NULL)
    this->pTrace->trace("in IniFileWx::load() filename=", filename);
  if (filename == NULL)
  {
    no_entries = -1L;
    goto zurueck;
  }
  ifd = ::fopen(filename, "r");
  if (ifd == NULL)
  {
    no_entries = -2L;
    goto zurueck;
  }
  this->origin_fn = filename;
  ::strcpy(section, "");
  if (this->pTrace != NULL)
    this->pTrace->trace("in IniFileWx::load() file OK");
  //
  while (1)
  {
    if (::fgets(buf, sizeof(buf), ifd) == NULL)
      break;
    if (this->pTrace != NULL)
      this->pTrace->trace("in IniFileWx::load() line=", buf);
    clean_buf(buf);
    if (::strlen(buf) <= 2)
      // line probably not correct (or empty)
      continue;
    if (buf[0] == '#')
		// comment line
		continue;
    if (buf[0] == ';')
		// comment line
		continue;
    
    if (buf[0] == '[')
    {
      // this is a section name - keep it
      caddr = ::strchr(buf, ']');
      if (caddr != 0) *caddr = 0;
      ::strcpy(section, &buf[1]);
      if (this->pTrace != NULL)
        this->pTrace->trace("in IniFileWx::load() section=", section);
      continue;
    }
    no_entries++;
    if (this->pTrace != NULL)
    {
      char aux[256];
      sprintf(aux, "%lu %s\n", no_entries, buf);
      this->pTrace->trace(aux);
    }   
    //
    // extract the key
    m = 0;
    ::strcpy(k, buf);
    for (n=0; n < sizeof(buf); n++)
    {
      if ((buf[n] == '\n') || (buf[n] == '\r'))
         break;
      if (buf[n] == 0)
        break;
      if (buf[n] == '=')
      {
        k[n] = 0;
        m = n + 1;
        break;
      }
    }
    if (m <= 0)
      continue;
    secStr = section;
    itStr  = k;
    valStr = &buf[m];
    ret = this->add(secStr, itStr, valStr);
    if (ret < 0)
    {
      if (this->pTrace != NULL)
        this->pTrace->trace("in IniFileWx::load() cannot add=", itStr);
      no_entries = -3L;
      goto zurueck;
    }
    if (this->pTrace != NULL)
    {
      this->pTrace->trace("in IniFileWx::load() added section=", secStr);
      this->pTrace->trace("in IniFileWx::load() added item=", itStr);
      this->pTrace->trace("in IniFileWx::load() added value=", valStr);
    }
  }    // end while 1
  //  
  zurueck:
  if (ifd != NULL)
    ::fclose(ifd);
  return no_entries;  // ok
}   // end IniFileWx::load()

// ---------------------------------------------------------------
// write the dictionary into a Windows like ini-File
// Returns: >0   if successful
//                < 0  if error
// Note: at the moment the resulting file hasn't the same format
//       as load-file!
int IniFileWx::write(const char *filename)
{
  FILE *ofd;
  wxString lastSection = "";
  wxString section;
  wxString key;
  wxString fullKey;
  int    flag;
  HUStringMap::iterator it;
  //
  if (filename == NULL)
    return 0;
  ofd = ::fopen(filename, "w");
  if (ofd == 0)
    return -1;
  //
  it = this->mmap.begin();
  while (it != this->mmap.end())
  {
    fullKey = (*it).first;
    flag    = 0;
    section = "";
    key     = "";
    for (unsigned int i=0; i < fullKey.length(); i++)
    {
       if (flag == 0)
         section = section + fullKey[i];
       else
         key = key + fullKey[i];
       if (fullKey[i] == ']')
         flag = 1;
    }
    if (lastSection != section)
      ::fprintf(ofd, "%s\n", (const char *)section.c_str());
    ::fprintf(ofd, "%s=%s\n", (const char *)key.c_str(), (const char *)((*it).second).c_str());
    it++;
    lastSection = section;
  }
  ::fclose(ofd);
  return 1;   // ok
}   // end IniFileWx::write()

// ---------------------------------------------------------------
int IniFileWx::write(FILE *ofd)
{
  HUStringMap::iterator it;
  //
  it = this->mmap.begin();
  while (it != this->mmap.end())
  {
    fprintf(ofd, "%s=%s\n", (const char *)((*it).first).c_str(), (const char *)((*it).second).c_str());
    it++;
  }
  fflush(ofd);
  return 1;   // ok
}   // end IniFileWx::write()

// ---------------------------------------------------------------
// add a key-value pair to the dictionary
// Returns: >0   if successful
//          ==0  if error
int IniFileWx::add(const wxString & section, 
                   const wxString & item, 
                   const wxString & value)
{
  int ret = 0;
  wxString key;
  wxString norm_s, norm_i;
  //
  norm_s = section.Lower();
  norm_i = item.Lower();
  key  = this->makeKey(norm_s, norm_i);
  this->mmap[key] = value;
  return ret;
}   // end IniFileWx::add()

// ---------------------------------------------------------------
// Retrieve the first value via the key
// the (combined) key is generated from section and item
// returns: a pointer to the value if key found
//          NULL if key not found
// Note: the section parameter may be 0 or an empty wxString
// Note: don't return here an internal address: this leads to a crash
//       We must return a new object or a static object
wxString IniFileWx::get(const wxString & section, const wxString & item)
{
  wxString   key;
  wxString   norm_s, norm_i;
  static wxString rets;   // must be static!!!
  //
  rets = "";
  norm_s = section.Lower();
  norm_i = item.Lower();
  key = this->makeKey(norm_s, norm_i);
  rets = this->mmap[key];
  return rets;
}   // end IniFileWx::get()

// ---------------------------------------------------------------
// Returns: -1 if nothing found!
int IniFileWx::getInt(const wxString &section, const wxString & item)
{
  int reti = -1;
  wxString result;
  //
  result = this->get(section, item);
  if (result.Length() > 0)
    reti = ::atoi((const char *)result.c_str());
  return reti;
}    // end IniFileWx::getInt()

// ---------------------------------------------------------------
int IniFileWx::replace(const wxString & section, 
                       const wxString & item, 
                       const wxString & value)
{
  int ret;
  wxString   norm_s, norm_i;
  //
  this->erase(section, item);
  ret = this->add(section, item, value);
  return ret;
}   // end IniFileWx::replace()

// ---------------------------------------------------------------
// Erase the values with the given key
// The (combined) key is generated from section and item
// Returns: >     if ok 
//          == 0  if error
// Note: the section parameter may be 0 or an empty wxString
int IniFileWx::erase(const wxString & section, const wxString & item)
{
  int    ret;
  wxString key;
  wxString   norm_s, norm_i;
  //
  norm_s = section.Lower();
  norm_i = item.Lower();
  key = this->makeKey(norm_s, norm_i);
  ret = (this->mmap).erase(key);
  return ret;
}   // end IniFileWx::erase()
