// module util.cpp
// Declares a set of utility methods
// Requires the STL
// Private Copyright Dr.E.Huckert 01-2003
//
// Version: 2003-12-16 all global functions added from module hu_utils.cpp
//  2006-11-15 extended versions getField() and get_field added
//             method ::trim() redesigned
//             method UTIL::trim() added
//  2008-02-26 include Dateien ueberarbeitet
//             milli_sleep() fuer UNIX ueberarbeitet
// 13.08.2009  Alle Routinen mit Aufruf time() und ftime() sind gefaehrlich
//             in real-time Anwendungen!!!!
// 10.08.2010  utf2isoLatin() hinzu
// 02.08.2014 wxString ersetzt durch wxString
// 20.11.2014 added Method UTIL__cvtFileName()

#include "util_hu.h"

// Constructor
UTIL::UTIL()
{
  this->logging = 0;
}

// ------------------------------------------------------------------
void UTIL::toupperBuf(char *buf)
{
  int n;
  for (n=0; buf[n] != 0; n++)
  {
    buf[n] = toupper(buf[n]);
  }
}   // end UTIL::toupperBuf()

// --------------------------------------------------------------------
void UTIL::tolowerBuf(char *buf)
{
  int n;
  for (n=0; buf[n] != 0; n++)
  {
    buf[n] = tolower(buf[n]);
  }
}   // end UTIL::tolowerBuf()

// -------------------------------------------------------------------
unsigned char UTIL::decode_hex(const char *src, int pos)
{
  unsigned char rval = 0;
  unsigned char ch   = 0;
  // first byte (nibble)
  ch = *(src + pos);
  if (ch <= '9') ch -= '0';
  else
  {
    if (ch <= 'F') ch = 10 + (ch - 'A');
    else           ch = 10 + (ch - 'a');
  }
  rval = ch << 4;
  //
  // second byte(nibble)
  ch = *(src + pos + 1);
  if (ch <= '9') ch -= '0';
  else
  {
    if (ch <= 'F') ch = 10 + (ch - 'A');
    else           ch = 10 + (ch - 'a');
  }
  rval = rval + ch;
  return rval; 
}   // end UTIL::decode_hex()

// -------------------------------------------------------------------
// convert to HEX notation
// The result may be incomplete if the buffer dest is too short!
wxString UTIL::bytesToHex(const char *bp, int lbp)
{
int n, m;
wxString rets;
char   aux[16];
  rets.reserve((lbp * 2) + 1);
  m = 0;
  for (n=0; n < lbp; n++)
  {
    sprintf(aux, "%02x", bp[n] & 0xff);
    rets.append(aux);
  }
  return rets;
}   // end UTIL::bytesToHex()

// ---------------------------------------------------------------------
// delivers the Epoch time in secs
long UTIL::getEpochTime(void)
{
  time_t   ltime;
  ltime = time(0);
  return (long)ltime;
}   // end UTIL::getEpochTime()

// ---------------------------------------------------------------------
// Vorsicht: liefert Wert in ms, nicht sec.!
long UTIL::getEpochTime_ms(void)
{
  time_t   ltime;
  struct timeb tb;
  //ltime = time(0);  // 13.08.2009 gefaehrlich in real-time Anwendungen da 2 -System-Calls 
  ftime(&tb);
  ltime = tb.time;    // 13.08.2009
  return (long) (ltime * 1000 + (int)tb.millitm);
}   // end UTIL::getEpochTime_ms()

// --------------------------------------------------------------------
// liefert Datum im Format TT.MM.JJJJ
wxString UTIL::getDate()
{
  time_t   ltime;
  struct tm *newtime;
  short  offset;
  char buf[64];
  time(&ltime);
  newtime = localtime(&ltime);
  offset  = 1900; 
  sprintf(buf,"%02d.%02d.%04d",
          (int)newtime->tm_mday,
          (int)newtime->tm_mon+1,
          (int)newtime->tm_year + offset);
  wxString aux(buf);
  return aux;
}   // end getDate()

// --------------------------------------------------------------------
// liefert Zeit im Format SS:MM:SS:mmm 
wxString UTIL::getTime()
{
  time_t   ltime;
  struct tm *newtime;
  struct timeb tb;
  char buf[64];
  time(&ltime);  // 13.08.2009 gefaehrlich: 2 systems calls (real time!!!!)
  newtime = localtime(&ltime);
  ftime(&tb);    // 13.08.2009 gefaehrlich: 2 systems calls (real time!!!!)
  sprintf(buf,"%02d:%02d:%02d.%03d",
          (int)newtime->tm_hour,
          (int)newtime->tm_min,
          (int)newtime->tm_sec,
          (int)tb.millitm);
  wxString aux(buf);
  return aux;
}   // end getTime()

// ---------------------------------------------------------------------
// liefert Datum im Format YYYY-MM-DD 
wxString UTIL::getAmericanDate()
{
  time_t   ltime;
  struct tm *newtime;
  short  offset;
  char   buf[64];
  time(&ltime);
  newtime = localtime(&ltime);
  offset  = 1900;
  sprintf(buf,"%04d-%02d-%02d",
          (int)newtime->tm_year + offset,
          (int)newtime->tm_mon+1,
          (int)newtime->tm_mday);
  wxString aux(buf);
  return aux;
}   // end getAmericanDate()

// ----------------------------------------------------------------------
// liefert Datum im Format TT.MM.JJJJ
// Input: beliebiger Zeitwert im Epoch Format 
wxString UTIL::makeDate(unsigned long epoch)
{
  wxString   s;
  struct tm *newtime;
  short  offset;
  char dest[64];  
  newtime = localtime((long *)&epoch);
  offset  = 1900;
  sprintf(dest,"%02d.%02d.%04d",
          (int)newtime->tm_mday,
          (int)newtime->tm_mon+1,
          (int)newtime->tm_year + offset);
  s = dest;
  return s;
}   // end makeDate()

// ------------------------------------------------------------------------
// liefert Datum im Format YYYY-MM-DD
// Input: beliebiger Zeitwert im Epoch Format 
wxString UTIL::makeAmericanDate(unsigned long epoch)
{
  wxString   s;
  struct tm *newtime;
  short  offset;
  char dest[64];  
  newtime = localtime((long *)&epoch);
  offset  = 1900;
  sprintf(dest,"%04d-%02d-%02d",
          (int)newtime->tm_year + offset,
          (int)newtime->tm_mon+1,
          (int)newtime->tm_mday);
  s = dest;
  return s;
}   // end makeAmericanDate()

// ------------------------------------------------------------------------
// strict format YYYY-MM-DD HH:MM:SS.000000000 here assumed
// convert this to a vanilla Epoch time
unsigned long UTIL::timestamp2long(const char *tstamp)
{
  struct tm zeit;
  int ll;
  ll = strlen(tstamp);
  memset((char *)&zeit, 0, sizeof(struct tm));
  //printf("tstamp=%s\n", tstamp);
  zeit.tm_year = atoi(&tstamp[0]) - 1900;
  //printf("tm_year=%d\n", zeit.tm_year);
  zeit.tm_mon  = atoi(&tstamp[5]) - 1;
  zeit.tm_mday = atoi(&tstamp[8]);
  if (ll > 10) 
  {
    zeit.tm_hour = atoi(&tstamp[11]);
    zeit.tm_min  = atoi(&tstamp[14]);
    zeit.tm_sec  = atoi(&tstamp[17]);
  }
  else
  {
    zeit.tm_hour = 0;
    zeit.tm_min  = 0;
    zeit.tm_sec  = 0;
  }
  return ((long)mktime(&zeit));
}  // end UTIL::timestamp2long()

// ---------------------------------------------------------------------
int UTIL::getNoFields(const  char   *buf, 
                      char   sep)        // field separator character  
{
  int    n, nof;
  nof    = 0;
  for (n=0; buf[n] != 0; n++)
    if (buf[n] == sep) 
      nof++;
  return nof;
}   // end UTIL::getNoFields()

// ---------------------------------------------------------------------
// Parse a string and return the desired field content
// returns: the extracted string 
//          the field is returned in string retbuf
// Note: buf is something like "field1|field2|field3|...|"
wxString UTIL::getField(const  char   *buf, 
                        int    no_field,   // from 0 on
                        char   sep)        // field separator character  
{
  wxString retbuf;
  int    n, nof;
  nof    = 0;
  retbuf.erase();
  //retbuf.reserve(256);
  //
  for (n=0; buf[n] != 0; n++)
  {
    if (buf[n] == sep)
    {
      nof++;
      if (nof > no_field) 
      {
        return retbuf;
      }
      //retbuf.erase();
    }
    else
    {
      if (nof == no_field)
        retbuf.append(buf[n]);
    }
  }   // for n ...
  //
  retbuf.erase();
  return retbuf;
}   // end UTIL::getField()

// -----------------------------------------------------------------------
// Parse a string and return the desired field content
// returns: the number of extracted characters
//          the returned content (retbuf) is zero terminated
int UTIL::getField(const char  *buf, 
                   int   no_field,   // from 0 on
                   char  *retbuf,   // return buffer
                   int   lretbuf,
                   char  sep)       // field separator character  
{
  int  m, n, nof;
  m   = 0;
  nof = 0;
  for (n=0; buf[n] != 0; n++)
  {
    if (buf[n] == sep)
    {
      retbuf[m] = 0;
      nof++;
      if (nof > no_field) 
      {
        return m;
      }
      m = 0;
    }
    else
    {
      if (m < (lretbuf - 1))
        retbuf[m++] = buf[n];
    }
  }   // for n ...
  return 0;
}   // end UTIL::getField()

// ---------------------------------------------------------------------
int UTIL::get_field(const char  *buf, 
                    int   no_field,   // from 0 on
                    char  *retbuf,   // return buffer
                    int   lretbuf,
                    char  sep)       // field separator character  
{
  return
    UTIL::getField(buf, no_field, retbuf, lretbuf, sep);
}

// ----------------------------------------------------------------------
// Replace all occurences of pattern1 by pattern2 in string buf
// Assumes that pattern1 and pattern2 have the same length!
wxString UTIL::replacePattern(const char *buf,
                            const char *pattern1,
                            const char *pattern2)
{
  wxString result;
  char *newbuf;
  int   m, n, l1, ll;
  result =  buf;
  l1 = strlen(pattern1);
  ll = strlen(buf) - l1;
  for (m=0; m <= ll; m++)
  {
    if (strncmp(&buf[m], pattern1, l1) == 0)
    {
      for (n=0; pattern2[n] != 0; n++)
        result.append(pattern2[n]);
    }
  }
  return result;
}   // end replacePattern()

// ----------------------------------------------------------------------
// eliminate unwanted characters from a buffer
void UTIL::cleanBuf(char *buf)
{
   int n;
   for (n=0; buf[n] != 0; n++)
   {
     if (buf[n] == '\n') buf[n] = 0;
     if (buf[n] == '\r') buf[n] = 0;
   }
}   // end cleanBuf()

// -----------------------------------------------------------------------
// Returns: <> 0 if string is wff
//             0 if string is not wff
int UTIL::checkString(const char *field,
                      int empty_allowed)
{
  int ll, ld, n;
  ll = strlen(field);
  if (ll == 0)
  {
    if (empty_allowed) return 1;
    else            return 0;
  }
  return 1;
}   // end UTIL::checkString()

// ------------------------------------------------------------------------
// Returns: <> 0 if date is wff
//             0 if date is not wff
int UTIL::checkDate(const char *field,
                    int empty_allowed)
{
  int ll, ld, n;
  ll = strlen(field);
  if (ll == 0)
  {
    if (empty_allowed) return 1;
    else            return 0;
  }
  ld = strlen("YYYY-MM-DD");
  if (ll != ld)
    return 0;
  for (n=0; n < ld; n++)
  {
    if (strchr("0123456789-", field[n]) == NULL)
      return 0;
  }
  wxString item = UTIL::getField(field, 0, '-');
  if (item.length() != 4)
    return 0;
  item = UTIL::getField(field, 1, '-');
  n = atoi(item.c_str());
  if (n < 1)
    return 0;
  if (n > 12)
    return 0;
  if (item.length() != 2)
    return 0;
  wxString aux = field;
  aux.append("-");
  item = UTIL::getField(aux.c_str(), 2, '-');
  n = atoi(item.c_str());
  if (n < 1)
    return 0;
  if (n > 31)
    return 0;
  //
  return 1;
}   // end UTIL::checkDate()

// -----------------------------------------------------------------------
// Returns: <> 0 if year is wff
//             0 if year is not wff
int UTIL::checkYear(const char *field,
                    int empty_allowed)
{
  int ll, ld, n;
  ll = strlen(field);
  if (ll == 0)
  {
    if (empty_allowed) return 1;
    else            return 0;
  }
  ld = strlen("YYYY");
  if (ll != ld)
    return 0;
  for (n=0; n < ld; n++)
  {
    if (strchr("0123456789", field[n]) == NULL)
      return 0;
  }
  return 1;
}   // end UTIL::checkYear()

// ------------------------------------------------------------------------
int UTIL::cvtInt(const wxString & s)
{
  long lv;
  s.ToLong(&lv);
  return (int)lv;
}   // end UTIL::cvtInt()

// ------------------------------------------------------------------------
wxString UTIL::cvtString(long lv)
{
  char aux[32];
  wxString s;
  ::sprintf(aux, "%ld", lv);
  s = aux;
  return s;
}   // end UTIL::cvtInt()

// -----------------------------------------------------------------------
// Returns: <> 0 if int is wff
//             0 if int is not wff
int UTIL::checkInt(const char *field,
                   int empty_allowed)
{
  int ll, ld, n;
  ll = strlen(field);
  if (ll == 0)
  {
    if (empty_allowed) return 1;
    else            return 0;
  }
  for (n=0; n < ll; n++)
  {
    if (strchr("0123456789", field[n]) == NULL)
      return 0;
  }
  return 1;
}   // end UTIL::checkInt()

// ------------------------------------------------------------------------
// Returns: <> 0 if float is wff
//             0 if float is not wff
int UTIL::checkFloat(const char *field,
                       int empty_allowed)
{
  int ll, ld, n;
  ll = strlen(field);
  if (ll == 0)
  {
    if (empty_allowed) return 1;
    else            return 0;
  }
  for (n=0; n < ll; n++)
  {
    if (strchr(".0123456789", field[n]) == NULL)
      return 0;
  }
  return 1;
}   // end UTIL::checkFloat()

// ----------------------------------------------------------------------
wxString UTIL::convertDate(const char *date,
                         const int outlength)
{
  wxString retdate;
  int n;
  retdate = "";
  for (n=0; n < outlength; n++)
  {
    if (date[n] == 0)
      break;
    retdate.append(date[n]);
  }
  return retdate;
}   // end UTIL::convertDate()

// ----------------------------------------------------------------------
wxString UTIL::convertFloat(const char *number,
                          const int outdigits)
{
  wxString retnumber;
  int n, flag, postcomma;
  retnumber = "";
  flag      = 0;
  postcomma = 0;
  for (n=0; ; n++)
  {
    if (number[n] == 0)
      break;
    retnumber.append(number[n]);
    if (number[n] == '.') 
    {
      flag = 1;
      continue;
    }
    if (flag)
    {
      postcomma++;
      if (postcomma >= outdigits)
        break;
    }
  }
  return retnumber;
}   // end UTIL::convertFloat()

// --------------------------------------------------------------
char *UTIL::trim(const char *buf)
{
  return ::trim((char *)buf);
}

// --------------------------------------------------------------------------
// Convert a long value to a variable-length value
// first bit of each byte set except the last byte
// Returns: the var.length value as a long value
//          the effective length in no_bytes
long UTIL::makeVarLength(long value, short *no_bytes)
{
long   retlong;
long   pattern;
int    ll, n, shift;
char   ch;
char   *dest;
  //
  dest = (char *)&retlong;
  // use only 28 bits (= 4 x 7 bits)
  value = value & 0x0fffffff;
  value = value << 4;
  pattern = 0xfe000000;
  shift   = 28 - 7;
  ll      = 0;
  for (n=0; n < 4; n++)
  {
    ch = (value & pattern) >> 24;
    ch = (ch >> 1) & 0x7f;
    value = value << 7;
    if (n < 3)
      ch = ch | 0x80;
    else
      // last byte of length
    ch = ch & 0x7f;
    *dest++ = ch;
    ll++;
   }
   *no_bytes = ll;
   return retlong;
}   // end UTIL::makeVarLength()

// --------------------------------------------------------------
// Transformiert UTF-8 2-Byte Code nach ISO-LATIN-1 1-Byte Kodierung
// Hinweis: intern muss unbedingt mit unsigned char gearbeitet werden!
wxString UTIL::eliminateQuotes(const wxString & inp)
{
  wxString out = "";
  int flag = 0;
  //
  for (int n=0; n < inp.Length(); n++)
  {
    if (inp[n] == '&')
    {
      flag = 1;
    }
    if (inp[n] == ';')
    {
      flag = 0;
      continue;
    }
    if (flag == 0)
      out.Append(inp[n]);
  }   // end for n...
  //
  return out;
}   // end UTIL::eliminateQuotes()

// --------------------------------------------------------------
// Transformiert UTF-8 2-Byte Code nach ISO-LATIN-1 1-Byte Kodierung
// Hinweis: intern muss unbedingt mit unsigned char gearbeitet werden!
wxString UTIL::utf2isoLatin(const unsigned char *pInp)
{
  wxString out;
  int n;
  char ch;
  out = "";
  if (pInp == NULL)
    return out;
  for (n=0; pInp[n] != 0; n++)
  {
    if (pInp[n] == 0xc3)  // Vorschaltzeichen fuer UTF-8
    {
      ch = '?';
      if (pInp[n+1] == 0) break;
      // switch ueber 2.Byte hinter Vorschaltcode
      switch (pInp[n + 1])
      {
        case 0xa9:
          ch = '';
          break;
        case 0xa8:
          ch = '';
          break;
        case 0x84:
          ch = '';
          break;
        case 0x96:
          ch = '';
          break;
        case 0x9c:
          ch = '';
          break;
        case 0xa4:
          ch = '';
          break;
        case 0xb6:
          ch = '';
          break;
        case 0xbc:
          ch = '';
          break;
        case 0xaa:
          ch = '';
          break;
        case 0xa0:
          ch = '';
          break;
        case 0x9f:
          ch = '';
          break;
        default:
          ch = '?';
          break;
      }
      out.append(ch);
      n++;
    }
    else
      out.Append(pInp[n]);
  }
  return out;
}   // end UTIL::utf2isoLatin()

// --------------------------------------------------------------------------
/* swap the byte order in a short word */
short UTIL::swapShort(short val)
{
char *s,*d;
short volatile i;
  d = (char *)&i;
  s = (char *)&val;
  *(d+1) = *s;
  *d     = *(s + 1);
  return i;
}   /* end swapShort() */

// --------------------------------------------------------------------------
/* swap the byte order in a long word */
long UTIL::swapLong(long val)
{
char *s,*d;
long volatile i;
  d = (char *)&i;
  s = (char *)&val;
  *(d+3) = *s;
  *(d+2) = *(s + 1);
  *(d+1) = *(s + 2);
  *(d+0) = *(s + 3);
  return i;
}   /* end swapLong() */

// ----------------- former module hu_utils.cpp ----------------
// include here for compatibility with older programs
// Private Copyright Dr.E.Huckert, Goldbach(Germany) 1997

void tolower_buf(const char *buf, char *dest, int ldest)
{
int n;
  for (n=0; buf[n] != 0; n++)
  {
    if (n < (ldest - 1))
    {
      dest[n]     = tolower(buf[n]);
      dest[n + 1] = 0;
    }
  }
}   // end tolower_buf()

// ---------------------------------------------------------------------
void tolower_buf(char *buf)
{
int n;
 for (n=0; buf[n] != 0; n++)
 {
   buf[n] = tolower(buf[n]);
 }
}   // end tolower_buf()

// -----------------------------------------------------------------------
void clean_buf(char *buf)
{
int n;
 for (n=0; buf[n] != 0; n++)
 {
   if (buf[n] == '\n') buf[n] = 0;
   if (buf[n] == '\r') buf[n] = 0;
 }
}   // end clear_buf()

// ---------------------------------------------------------------------
// Eliminates leading and trailing whitespace
// Allocates a new buffer!
// Returns: the new buffer
char *trim(char *buf)
{
int n, ll;
char *nbuf;
 // Startposition feststellen
 int start = 0;
 for (n=0; buf[n] != 0; n++)
 {
   if (! ((buf[n] == ' ') || (buf[n] == '\t')))
   {
     start = n;
     break;
   }
 }
 // Endeposition feststellen
 ll = strlen(buf);
 int ende = start;
 for (n=ll-1; n >= start; n--)
 {
   if (! ((buf[n] == ' ') || (buf[n] == '\t')))
   {
     ende = n;
     break;
   }
 }
 // Zielpuffer anlegen und fuellen
 nbuf = new char[ende - start + 2];
 strncpy(nbuf, &buf[start], ende - start + 1);
 nbuf[ende - start + 1] = 0;
 return nbuf;
}   // end trim()

// -------------------------------------------------------------------------
// convert to HEX notation
// The result may be incomplete if the buffer dest is too short!
void bytes_to_hex(char *bp, int lbp, char *dest, int ldest)
{
  int n, m;
  m = 0;
  for (n=0; n < lbp; n++)
  {
    if (m >= (ldest - 2))
      break;
    sprintf(&dest[m], "%02x", bp[n] & 0xff);
    m += 2;
  }
}   // end bytes_to_hex()

// -----------------------------------------------------------------------
// liefert Zeit im Format SS:MM:SS:mmm 
void get_time(char *dest)
{
  time_t   ltime;
  struct tm *newtime;
  struct timeb tb;
  //
  time(&ltime);  // 13.08.2009 gefaehrlich: 2 systems calls (real time!!!!)
  newtime = localtime(&ltime);
  ftime(&tb);    // 13.08.2009 gefaehrlich: 2 systems calls (real time!!!!)
  sprintf(dest,"%02d:%02d:%02d.%03d",
          (int)newtime->tm_hour,
          (int)newtime->tm_min,
          (int)newtime->tm_sec,
          (int)tb.millitm);
}   // end get_time()

// -----------------------------------------------------------------------
// liefert Zeit im Format SS:MM:SS.mmm 
// Input: beliebiger Zeitwert im Epoch Format 
void make_time(long epoch, char *dest)
{
  time_t   ltime;
  struct tm *newtime;
  struct timeb tb;
  //
  ftime(&tb);
  time(&ltime);
  //newtime = gmtime(&epoch);
  newtime = localtime(&epoch);
  sprintf(dest,"%02d:%02d:%02d.%03d",
          (int)newtime->tm_hour,
          (int)newtime->tm_min,
          (int)newtime->tm_sec,
          (int)tb.millitm);
}   // end make_time()

// -------------------------------------------------------------------------
// liefert Datum im Format TT.MM.JJJJ
// Input: beliebiger Zeitwert im Epoch Format 
void make_date(long epoch, char *dest)
{
  time_t   ltime;
  struct tm *newtime;
  short  offset;
  //newtime = gmtime(&epoch);
  newtime = localtime(&epoch);
  offset  = 1900;
  sprintf(dest,"%02d.%02d.%04d",
          (int)newtime->tm_mday,
          (int)newtime->tm_mon+1,
          (int)newtime->tm_year + offset);
}   // end make_date()

// -------------------------------------------------------------------------
// liefert Datum und Zeit im Format YYYY-MM-DD HH:MM:SS
// Input: beliebiger Zeitwert im Epoch Format 
void make_date_time(long epoch, char *dest)
{
  time_t   ltime;
  struct tm *newtime;
  short  offset;
  struct timeb tb;
  //
  ftime(&tb);
  //newtime = gmtime(&epoch);
  newtime = localtime(&epoch);
  offset  = 1900;
  //sprintf(dest,"%04d-%02d-%02d %02d:%02d:%02d.%03d",
  sprintf(dest,"%04d-%02d-%02d %02d:%02d:%02d",
          (int)newtime->tm_year + offset,
          (int)newtime->tm_mon+1,
          (int)newtime->tm_mday,
          (int)newtime->tm_hour,
          (int)newtime->tm_min,
          (int)newtime->tm_sec);
          //(int)tb.millitm);
}   // end make_date_time()

// -------------------------------------------------------------------------
// liefert Zeit im Epoch Format + 3 Stellen Millisekunden 
void get_epoch_time(char *dest)
{
  time_t   ltime;
  struct timeb tb;
  //
  ltime = time(0);
  ftime(&tb);
  sprintf(dest,"%ld.%03d",
          (long)ltime,
          (int)tb.millitm);
}   // end get_epoch_time()

// -------------------------------------------------------------------------
long get_epoch_time(void)
{
  time_t   ltime;
  struct timeb tb;
  ltime = time(0);
  ftime(&tb);
  return (long) (ltime * 1000 + (int)tb.millitm);
}   // end get_epoch_time()

// --------------------------------------------------------------------------
long get_epoch_time_ms(void)
{
  time_t   ltime;
  struct timeb tb;
  ltime = time(0);
  ftime(&tb);
  return (long) (ltime * 1000 + (int)tb.millitm);
}   // end get_epoch_time_ms()

// --------------------------------------------------------------------------
// liefert Datum im Format TT.MM.JJJJ
void get_date(char *dest)
{
  time_t   ltime;
  struct tm *newtime;
  short  offset;
  //
  time(&ltime);
  newtime = localtime(&ltime);
  offset  = 1900;
  sprintf(dest,"%02d.%02d.%04d",
          (int)newtime->tm_mday,
          (int)newtime->tm_mon+1,
          (int)newtime->tm_year + offset);
}   // end get_date()

// -------------------------------------------------------------------------
// liefert Datum im Format YYYY-MM-DD 
void get_american_date(char *dest)
{
  time_t   ltime;
  struct tm *newtime;
  short  offset;
  //
  time(&ltime);
  newtime = localtime(&ltime);
  offset  = 1900;
  sprintf(dest,"%04d-%02d-%02d",
          (int)newtime->tm_year + offset,
          (int)newtime->tm_mon+1,
          (int)newtime->tm_mday);
}   // end get_american_date()

//-------------------------------------------------------------------------
// Parse a string and return the desired field content
// returns: the number of extracted characters
//          the returned content (retbuf) is zero terminated
int get_field(char  *buf, 
              int   no_field,   // from 0 on
              char  *retbuf,   // return buffer
              int   lretbuf,
              char  sep)       // field separator character  
{
int  m, n, nof;
  m   = 0;
  nof = 0;
  for (n=0; buf[n] != 0; n++)
  {
    if (buf[n] == sep)
    {
      retbuf[m] = 0;
      nof++;
      if (nof > no_field) 
      {
        return m;
      }
      m = 0;
    }
    else
    {
      if (m < (lretbuf - 1))
        retbuf[m++] = buf[n];
    }
  }   // for n ...
  return 0;
}   // end ::get_field()

// ------------------------------------------------------------------------
// Returns <> 0 if both given dates are the same day of year
// strict format YYYY-MM-DD HH:MM:SS.000000000 here assumed
// (as obtained by get_time)
// Note: this simple version doesn't care for leap years!!!
int isSameDay(const char *d1, const char *d2)
{
  if ((d1 == 0) || (d2 == 0))
    return 0;
  if (strlen(d1) < 10) 
    return 0;
  if (strlen(d2) < 10) 
    return 0;
  // compare month
  if (strncmp(d1 + 5, d2 + 5, 2) != 0)
    return 0;
  // compare day
  if (strncmp(d1 + 8, d2 + 8, 2) != 0)
    return 0;
  // day and month are identical
  return 1;
}   // end isSameDay()

// -------------------------------------------------------------------------
// Parse a string and return the address of the desired field 
char *get_field_address(char  *buf, 
                        int   no_field,   // from 0 on
                        char  sep)       // field separator character  
{ 
int  m, n, nof;
  if (no_field == 0)
    return buf;
  m   = 0;
  nof = 0;
  for (n=0; buf[n] != 0; n++)
  {
    if (buf[n] == sep)
    {
      nof++;
      if (nof >= no_field) 
      {
        return &buf[n+1];
      }
    }
  }   // for n ...
  return NULL;
}   // end ::get_field_address()

// ----------------------------------------------------------
// replace a (numerical) fieldwith value new_content
// use no_chars positions for the replacement
void replace_field(char *buf, 
                    int no_field,
                    int new_content,
                    int no_chars)   
{
int m, n, nof, last_pos;
char xbuf[10];
  sprintf(xbuf, "%04d", new_content);
  nof      = 0;
  last_pos = 0;
  for (n=0; buf[n] != 0; n++)
  {
    if (buf[n] == EVENT_FIELD_SEP)
    {
      nof++;
      if (nof > no_field) 
      {
        for (m=0; m < no_chars; m++)
        {
          buf[last_pos + m + 1] = xbuf[m];
        }
        break;
      }
      else
        last_pos = n;
    }
  }
  return;
}   // end replace_field()

// ----------------------------------------------------------
// Replace a field with value new_content
// The new content may not be longer than the old content!
void replace_field(char *buf, 
                   int no_field,
                   char *new_content,
                   char sep) 
{
int   n;
char *caddr;
  caddr = get_field_address(buf, no_field, sep);
  if (caddr == NULL)
    return;
  for (n=0; new_content[n] != 0; n++)
  {
    if (caddr[n] == sep) break;
    caddr[n] = new_content[n];
  }
  return;
}   // end replace_field()

// ----------------------------------------------------------
void second_sleep(int no_sec)
{
#ifdef WIN32
  Sleep(no_sec * 1000);
#else
  sleep(no_sec);
#endif
}

// ----------------------------------------------------------
// strict format YYYY-MM-DD HH:MM:SS.000000000 here assumed
// convert this to a vanilla Epoch time
long timestamp2long(char *tstamp)
{
struct tm zeit;
  memset((char *)&zeit, 0, sizeof(struct tm));
  //printf("tstamp=%s\n", tstamp);
  zeit.tm_year = atoi(&tstamp[0]) - 1900;
  //printf("tm_year=%d\n", zeit.tm_year);
  zeit.tm_mon  = atoi(&tstamp[5]) - 1;
  zeit.tm_mday = atoi(&tstamp[8]);
  zeit.tm_hour = atoi(&tstamp[11]);
  zeit.tm_min  = atoi(&tstamp[14]);
  zeit.tm_sec  = atoi(&tstamp[17]);
  return ((long)mktime(&zeit));
}  // end timestamp2long()

// ----------------------------------------------------------
// sleep a number of milliseconds
int milli_sleep(int millis)
{
int ret;
  ret = 0;
#ifdef WIN32
  ::Sleep(millis);
#else
  /*
   // Hinweis: nanosleep() laeuft unter LINUX nicht sauber
    struct pollfd pfd[1];
    pfd[0].fd      = 0;
    pfd[0].events  = 0;
    pfd[0].revents = 0;
    ret            = poll(pfd, 1, millis);  
  */
  ret = usleep((long)millis * 1000L);
#endif
  return ret;
}   // end milli_sleep()

// ----------------------------------------------------------
// parse a candidate ini-file item 
// returns: <> 0 if ok
int parse_ini_item(char *src,
                   char *sect, int lsect,
                   char *item, int litem,
                   char *value, int lvalue) 
{
int n, m, state;
  m     = 0;
  state = 0;
  for (n=0; src[n] != 0; n++)
  {
    if (src[n] == '[') continue;
    if (src[n] == ']')
    {
      sect[m] = 0;
      m = 0;
      state = 1;
      continue;
    } 
    if (src[n] == '=')
    {
      m = 0;
      state = 2;
      continue;
    }
    if (state == 0)
    {
      if (m < lsect)
        sect[m++] = src[n];
    }
    if (state == 1)
    {
      if (m < litem)
        item[m++] = src[n];
    }
    if (state == 2)
    {
      if (m < lvalue)
        value[m++] = src[n];
    }
  }   // end for n...
  if ((strlen(sect) == 0) || (strlen(item) == 0) || (strlen(value) == 0))
    return 0;
  return 1;  
}   // end parse_ini_item()

// ----------------------------------------------------------
// rtrim
// trims rightpadded space chars
char *rtrim(char *buf) 
{
char *src = buf;
  // walk to the end of the string
  while (*src) src++;
  // set spaces to \0
  src--;
  while (src >= buf) 
  {
    if (isspace(*src)) *src = '\0';
    src--;
  }
  return buf;
}   // end rtrim()

// ----------------------------------------------------------
int is_empty(char *buf)
{
  int empty, n;
  empty = 1;
  for (n=0; buf[n] != 0; n++)
  {
    if (! isspace(buf[n]))
    {
      empty = 0;
      break;
    }
  }
  return empty;
}   // end is_empty()

// ----------------------------------------------------------
// This is rather a truncation than a rounding!
void round_float(char *buf, int no_digits)
{
int dec_point = 0;
int n, dig;
  for (n=0; buf[n] != 0; n++)
  {
    if (buf[n] == '.')
    {
      dec_point = 1;
      dig = 0;
      continue;
    }
    if (dec_point)
    {
      dig++;
      if (dig > no_digits)
      {
        // truncate here
        buf[n] = 0;
        break;
      }
    }
  }   // end for n...
}   // end round_float()

// ----------------------------------------------------------
// Replace all occurences of pattern1 by pattern1 in string buf
// Assumes that pattern1 and pattern2 have the same length!
// Returns a new buffer, i.e. the input buffer is not changed!
char *replace_pattern(const char *buf,
                      const char *pattern1,
                      const char *pattern2)
{
char *newbuf;
int   m, n, l1, ll;
  newbuf = new char[strlen(buf) + 1];
  strcpy(newbuf, buf);
  l1 = strlen(pattern1);
  ll = strlen(buf) - l1;
  for (m=0; m <= ll; m++)
  {
    if (strncmp(&buf[m], pattern1, l1) == 0)
    {
      for (n=0; pattern2[n] != 0; n++)
        newbuf[m+n] = pattern2[n];
    }
  }
  return newbuf;
}   // end replace_patterns()

// ----------------------------------------------------------
void eliminate_string(char *str, char *pattern)
{
char *caddr;
char *nbuf;
int  ll;
  nbuf = new char[strlen(str) +1];
  ll   = strlen(pattern);
  while ((caddr = strstr(str, pattern)) != 0)
  {
    strcpy(nbuf, caddr + ll);
    *caddr = 0;
    strcat(str, nbuf);
  }
  delete nbuf;
  return;
} // end eliminate_string()

// ----------------------------------------------------------
// corresponds to basename() under UNIX
// Allocates and returns a new buffer!
// Ex.: returns "file" "for c:\xyz\file.txt"
char *baseName(const char *path)
{
int  n, ll, p, start;
char *retbuf;
  ll = strlen(path);
  retbuf = new char[ll + 1];
  start = 0;
  p     = -1;
  for (n=ll - 1; n >= 0; n--)
  {
    if (path[n] == '.')  p = n;
    if ((path[n] == '\\') || (path[n] == '/')) 
    {
      start = n + 1;
      break;
    }
  }
  strcpy(retbuf, &path[start]);
  if (p > 0)
    retbuf[p - start] = 0;
  return retbuf;
}   // end baseName()

// ----------------------------------
// Zeichnet (pseudo) einen Punkt
void SetPixel(int x, int y)
{
  printf("%d %d\n", x, y);
}   // end SetPixel()

// --------------------------------------------
int UTIL::bresenham_sgn(int x)
{
  return (x > 0) ? 1 : (x < 0) ? -1 : 0;
}   // end UTIL::bresenham_sgn()
 
// --------------------------------------------------------------
// Returns: Pointer auf short-Array, der fuer jeden X-Wert der 
// (Pseudo)-gezeichneten Kurve den Y-Wert enthaelt.
// Im Rueckgabearray sind die X-Werte (=Index) ab 0 normiert!!!
// Grundannahme: xstart, xend, ystart, yend >= 0 !!!
//               xend >= xstart !!!
// Der gelieferte Pointer muss extern ueber delete freigegeben werden!
// Praktisch nur fuer vorwiegend horizontale Linien brauchbar
// Quelle: aus Wikipedia
short * UTIL::bresenham(int xstart, int ystart, int xend, int yend)
{
  int x, y, t, dx, dy, incx, incy, pdx, pdy, ddx, ddy, es, el, err;
  short *retPtr = NULL;
  //
  if ((xend < 0) || (xstart < 0))
    return NULL;
  if (xend < xstart)
    return NULL;
  // Entfernung in beiden Dimensionen berechnen
   dx = xend - xstart;
   dy = yend - ystart;
   retPtr = new short[dx + 2];
   // Vorzeichen des Inkrements bestimmen 
   incx = UTIL::bresenham_sgn(dx);
   incy = UTIL::bresenham_sgn(dy);
   if (dx < 0) dx = -dx;
   if (dy < 0) dy = -dy;
   /// feststellen, welche Entfernung grer ist 
   if (dx > dy)
   {
      /* x ist schnelle Richtung */
      pdx = incx; pdy = 0;    /* pd. ist Parallelschritt */
      ddx = incx; ddy = incy; /* dd. ist Diagonalschritt */
      es  = dy;   el = dx;   /* Fehlerschritte schnell, langsam */
   } 
   else
   {
      /* y ist schnelle Richtung */
      pdx=0;    pdy=incy; /* pd. ist Parallelschritt */
      ddx=incx; ddy=incy; /* dd. ist Diagonalschritt */
      es =dx;   el =dy;   /* Fehlerschritte schnell, langsam */
   }
 
   // Initialisierungen vor Schleifenbeginn 
   x   = xstart;
   y   = ystart;
   err = el / 2;
   retPtr[x - xstart] = y;  // SetPixel(x,y);
   
   // Pixel berechnen 
   for (t=0; t < el; ++t) /* t zaehlt die Pixel, el ist auch Anzahl */
   {
      // Aktualisierung Fehlerterm 
      err -= es; 
      if (err < 0)
      {
          // Fehlerterm wieder positiv (>=0) machen 
          err += el;
          // Schritt in langsame Richtung, Diagonalschritt 
          x += ddx;
          y += ddy;
      } 
      else
      {
          // Schritt in schnelle Richtung, Parallelschritt 
          x += pdx;
          y += pdy;
      }
      retPtr[x - xstart] = y; // SetPixel(x,y);
   }
   //
   retPtr[xend - xstart + 1] = -1; // Endemarkierung
   return retPtr;
} // end UTIL::bresenham() 

// --------------------------------------------
// Convert a filename in standard UNIX syntax to Windows syntax
wxString UTIL::cvtFileName(const wxString srcFn)
{
  wxString outFn = "";
  //
  for (int n=0; n < srcFn.Length(); n++)
  {
    if (srcFn[n] == '/')
    {
#ifdef WIN32
      outFn.Append('\\');
#else
      outFn.Append('/');
#endif
    }
    else
      outFn.Append(srcFn[n]);
  }
  return outFn;
}   // end  UTIL::cvtFileName()

