// module util_hu.h
// Declares a set of utility methods
// Private Copyright Dr.E.Huckert 01-2003
//
// Versionen:
// 2003-12-16 all global functions added from module hu_utils.cpp
// 2010-02-15 Uses exclusively wxString as wxString class
// 10.08.2010 utf2isoLatin() hinzu
// 02.08.2014 wxString ersetzt durch wxString

#ifndef UTIL_DEFINED
#define UTIL_DEFINED

#include <stdlib.h>
#include <time.h>
#include <sys/timeb.h>

#ifdef WIN32
#include <windows.h>
//#include "wx/msw/winundef.h" // 21.08.2014 muss sein wg. Yield()
#else
#include <unistd.h>
#endif
#include <wx/string.h>

//using namespace std;

class UTIL
{
private:
  int logging;
public:
  UTIL();
  void setLogging(int flag)
  { this->logging = flag; };
  static wxString getField(const char *buf, int no_field, char  sep);  
  static int      getField(const char *buf, int no_field, 
                         char *dest, int ldest, char  sep);  
  static int      get_field(const char *buf, int no_field, 
                         char *dest, int ldest, char  sep);  
  static int      getNoFields(const char *buf, char  sep); 
  static char     *trim(const char *buf);  
  static wxString getDate();  
  static wxString getAmericanDate();  
  static wxString getTime();  
  static long     getEpochTime(void);
  static long     getEpochTime_ms(void);
  static wxString makeDate(unsigned long epoch);
  static wxString makeAmericanDate(unsigned long epoch);
  static unsigned long timestamp2long(const char *ts);
  static wxString replacePattern(const char *buf, 
                               const char *patt1, const char *patt2);
  static void   cleanBuf(char *buf);
  static int    checkInt(const char *buf, int empty_flag);
  static int    checkString(const char *buf, int empty_flag);
  static int    checkDate(const char *buf, int empty_flag);
  static int    checkYear(const char *buf, int empty_flag);
  static int    checkFloat(const char *buf, int empty_flag);
  static wxString convertDate(const char *date, const int outlength);
  static wxString convertFloat(const char *number, const int outlength);
  static void   tolowerBuf(char *buf);
  static void   toupperBuf(char *buf);
  static wxString bytesToHex(const char *buf, int lbuf);
  static wxString utf2isoLatin(const unsigned char *pInp);
  static wxString eliminateQuotes(const wxString & inp);
  static int     cvtInt(const wxString & s);
  static wxString cvtString(long lval);
  static long    swapLong(long val);
  static short   swapShort(short val);
  static long    makeVarLength(long value, short *no_bytes);
  static int     bresenham_sgn(int x);
  static short * bresenham(int xstart, int ystart, 
                           int xend, int yend);
  static wxString cvtFileName(const wxString fn);
  static unsigned char decode_hex(const char *p, int pos);
};

// ---------------------- former module hu_utils.h --------------
// These are essentially C (not C++) routines
// Private Copyright Dr.E.Huckert, Goldbach(Germany) 1997
// Version 2.0 2002-12-08 function bytes_to_hex() added

#define EVENT_FIELD_SEP      (char)'|'

// eliminiert "Schmutz"-Zeichen wie LF, CR
void clean_buf(char *buf);
char *trim(char *buf);
// translate to lower case letters (ASCII chars only!!!) 
void tolower_buf(char *buf);
// translate to lowercase in a sep. buffer (ASCII chars only!!!) 
void tolower_buf(const char *buf, char *dest, int ldest);
// translate a sequence of bytes into a hex string
void bytes_to_hex(char *bp, int lbp, char *dest, int ldest);
// liefert Zeit im Format SS:MM:SS:mmm 
void get_time(char *dest);
// Erzeugt Datum im Format YYYY-MM-DD aus einem
// beliebigen Epoch Wert
void make_date(long epoch, char *dest);
// Erzeugt Zeit im Format YYYY:MM:DD aus einem
// beliebigen Epoch Wert
void make_time(long epoch, char *dest);
// Erzeugt Datum + Zeit im Format "YYYY:MM:DD HH:MM:SS.mmm" aus einem
// beliebigen Epoch Wert
void make_date_time(long epoch, char *dest);
// liefert Zeit im Epoch Format + 3 Stellen Millisekunden 
void get_epoch_time(char *dest);
// liefert Zeit im Epoch Format + 3 Stellen Millisekunden 
long get_epoch_time(void);
long get_epoch_time_ms(void);
// liefert Datum im Format TT.MM.JJJJ
void get_date(char *dest);
// liefert Datum im Format YYYY-MM-DD 
void get_american_date(char *dest);
// replace a (numerical) fieldwith value new_content
// use no_chars positions for the replacement
void replace_field(char *buf, 
                    int no_field,
                    int new_content,
                    int no_chars); 
// Replace a field with value new_content
// The new content may not be longer than the old content!
void replace_field(char *buf, 
                   int no_field,
                   char *new_content,
                   char sep); 
// parse a candidate ini-file item 
// returns: <> 0 if ok
int parse_ini_item(char *src,
                   char *sect, int lsect,
                   char *item, int litem,
                   char *value, int lvalue);
void second_sleep(int nosec);
int  milli_sleep(int millis);
long timestamp2long(char *ts);
int  isSameDay(const char *xdate, const char *ydate);
// trims right padded space
char *rtrim(char *buf); 
char *replace_pattern(const char *buf, const char *patt1, const char *patt2);
int  is_empty(char *buf);
void eliminate_string(char *buf, char *pattern);
char *baseName(const char *path);
#endif

